
errors = {
    "421-connection-timeout":                                   "421 Connection timeout",
    "421-excessive-amounts-of-data":                            "421 Excessive amounts of data sent to server.",
    "451-please-try-again-later":                               "451 Please try again later.",
    "500-line-too-long":                                        "500 Line too long",
    "501-ehlo-invalid-domain-address":                          "501 EHLO Invalid domain address.",
    "502-turn-disallowed":                                      "502 TURN disallowed",
    "502-unimplemented-command":                                "502 Unimplemented command",
    "502-use-helo-ehlo-first":                                  "502 Use HELO/EHLO first",
    "502-vrfy-disallowed":                                      "502 VRFY disallowed",
    "503-issue-reset-to-start-over":                            "503 Issue a reset if you want to start over",
    "503-must-have-sender-and-recipient-first":                 "503 Must have sender and recipient first.",
    "503-must-have-sender-first":                               "503 Must have sender first.",
    "504-authentication-type-not-specified":                    "504 Authentication type not specified.",
    "530-smtp-authentication-is-required":                      "530 SMTP authentication is required.",
    "535-authentication-failed-restarting":                     "535 Authentication failed. Restarting authentication process.",
    "550-a-valid-address-is-required":                          "550 A valid address is required",
    "550-account-is-not-active":                                "550 Account is not active.",
    "550-alias-is-not-active":                                  "550 Alias is not active.",
    "550-blocked-by-spf":                                       "550 Blocked by SPF",
    "550-delivery-is-not-allowed-to-this-address":              "550 Delivery is not allowed to this address",
    "550-distribution-list-is-not-active":                      "550 Distribution list is not active.",
    "550-domain-has-been-disabled":                             "550 Domain has been disabled.",
    "550-invalid-syntax":                                       "550 Invalid syntax. Syntax should be MAIL FROM:<userdomain>[crlf]",
    "550-login-credentials-no-longer-valid":                    "550 Login credentials no longer valid. Please re-authenticate.",
    "550-mail-server-configuration-error":                      "550 Mail server configuration error. Too many recursive forwards.",
    "550-not-authorized":                                       "550 Not authorized.",
    "550-recipient-not-in-route-list":                          "550 Recipient not in route list.",
    "550-sender-address-must-be-specified":                     "550 Sender address must be specified.",
    "550-sender-domain-does-not-have-any-mx-records":           "550 Sender domain does not have any MX records.",
    "550-the-address-is-not-valid":                             "550 The address is not valid.",
    "550-the-host-name-specified-in-helo-does-not-match-ip":    "550 The host name specified in HELO does not match IP address.",
    "550-unknown-user":                                         "550 Unknown user",
    "550-other-error-message":                                  "550 <Other error message>",
    "552-message-size-exceeds-fixed-maximum":                   "552 Message size exceeds fixed maximum message size. Size: x KB, Max size: y KB",
    "554-rejected":                                             "554 Rejected",
    "554-your-message-was-received-but-could-not-be-saved":     "554 Your message was received but it could not be saved. Please retry later.",
    "554-rejected-containing-bare-lfs":                         "554 Rejected - Message containing bare LF's.",
    "554-rejected-no-data-saved":                               "554 Rejected - No data saved.",
    "554-rejected-other":                                       "554 Rejected - <other error message>",
    "554-tagged-as-spam-by-spamassassin":                       "554 Tagged as Spam by SpamAssassin",
    "550-mailbox-unavailable":                                  "550 Mailbox unavailable",
    "550-sender-verification-failed":                           "550 Sender verification failed",
}



import subprocess
import hitchserve
import sys
import os

# TODO: Create email object to signify email, address and attachment.
# TODO: "Wait until email arrives" function.
# TODO: Fix the recipient object conundrum.

class HitchSMTPService(hitchserve.Service):
    def __init__(self, python=sys.executable, port=10025, needs=None, **kwargs):
        kwargs['command'] = [python, "-u", "-m", "hitchsmtp.smtp", "--port", str(port)]
        kwargs['log_line_ready_checker'] = lambda line: "SMTP Server running" in line
        super(HitchSMTPService, self).__init__(**kwargs)



"""Mock SMTP Server."""
from hitchsmtp import smtperrors
import asyncore
import optparse
import signal
import email
import smtpd
import json
import sys
import re


class MockSMTPServer(smtpd.SMTPServer):
    """Mock SMTP server."""
    def __init__(*args, **kwargs):
        smtpd.SMTPServer.__init__(*args, **kwargs)

    def process_message(self, host, email_from, email_to, data):
        """Parse SMTP message and log it to the console as JSON."""
        parsed_message = email.Parser.Parser().parsestr(data)
        links_regex = re.compile(r"(https?://\S+)")

        if parsed_message.is_multipart():
            payload = []
            links = []
            for message in parsed_message.get_payload():
                payload_dict = dict(message)
                payload_dict['filename'] = message.get_filename()
                payload_dict['content'] = message.get_payload(decode=True)
                payload_dict['links'] = re.findall(links_regex, message.get_payload(decode=True))
                links = links + payload_dict['links']
                payload.append(payload_dict)
        else:
            payload = parsed_message.get_payload()
            links = re.findall(links_regex, payload)

        header_from = parsed_message.get('From')
        header_to = parsed_message.get('To')
        email_regex = re.compile(r"^(.*?)\<(.*?)\>$")

        if header_from:
            header_from_name = email_regex.match(header_from).group(1).strip() if email_regex.match(header_from) else None
            header_from_email = email_regex.match(header_from).group(2) if email_regex.match(header_from) else None
        else:
            header_from_name = header_from_email = None

        if header_to:
            header_to_name = email_regex.match(header_to).group(1).strip() if email_regex.match(header_to) else None
            header_to_email = email_regex.match(header_to).group(2) if email_regex.match(header_to) else None
        else:
            header_to_name = header_to_email = None


        dict_message = {
            'sent_from': email_from,
            'sent_to': email_to,
            'header_from': header_from,
            'header_to': header_to,
            'header_from_name': header_from_name,
            'header_to_name': header_to_name,
            'header_from_email': header_from_email,
            'header_to_email': header_to_email,
            'subject': parsed_message.get('Subject'),
            'date': parsed_message.get('Date'),
            'contenttype': parsed_message.get_content_type(),
            'multipart': parsed_message.is_multipart(),
            'payload': payload,
            'links': links,
        }
        sys.stdout.write(json.dumps(dict_message))
        sys.stdout.write('\n')
        sys.stdout.flush()

        if len(email_to) > 0:
            if email_to[0].endswith("@smtperrors.com"):
                name = email_to[0].replace("@smtperrors.com", "")
                if name in smtperrors.errors:
                    return smtperrors.errors[name]
                else:
                    raise Exception("{0} was not found in the list of SMTP errors.")


def server(port_number=10025):
    """Python library interface."""
    if port_number < 1024:
        sys.stderr.write("WARNING: Using a port below 1024 to run test Internet services"
                         " on is normally prohibited for non-root users, and usually inadvisable.\n\n")
        sys.stderr.flush()

    sys.stdout.write("SMTP Server running\n")
    sys.stdout.flush()

    smtp_server = MockSMTPServer(('localhost', port_number), None)

    def signal_handler(signal, frame):
        print('')
        smtp_server.close()
        sys.exit(0)
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)

    asyncore.loop()


def main():
    """CLI interface."""
    parser = optparse.OptionParser()
    parser.add_option("-p", "--port", type="int", dest="smtp_port", default=10025,
                      help="Specify the port number for the mock SMTP server to run on (default: 10025).")

    options, _ = parser.parse_args(sys.argv[1:])
    server(port_number=options.smtp_port)


if __name__ == '__main__':
    main()
